#ifndef POINT_H
#define POINT_H

#include <cmath>

const double point_h_pi = 3.1415926535897932384626433832795;
const double point_h_tolerance = 0.001;

class point {
public:
	point() : x(0.0), y(0.0) {}
	point(double x, double y) : x(x), y(y) {}

	point& operator+(point other) {
		this->x += other.x;
		this->y += other.y;
		return *this;
	}

	void operator+=(point other) {
		this->x += other.x;
		this->y += other.y;
	}

	point& operator-(point other) {
		this->x -= other.x;
		this->y -= other.y;
		return *this;
	}

	void operator-=(point other) {
		this->x -= other.x;
		this->y -= other.y;
	}

	point& operator*(double multiplier) {
		this->x *= multiplier;
		this->y *= multiplier;
		return *this;
	}

	void operator*=(double multiplier) {
		this->x *= multiplier;
		this->y *= multiplier;
	}

	point& operator/(double divisor) {
		this->x /= divisor;
		this->y /= divisor;
		return *this;
	}

	void operator/=(double divisor) {
		this->x /= divisor;
		this->y /= divisor;
	}

	bool operator==(point other) {
		return (std::abs(this->x - other.x) < point_h_tolerance) &&
			(std::abs(this->y - other.y) < point_h_tolerance);
	}

	bool operator!=(point other) {
		return !operator==(other);
	}

	//pass unit circle angle to find offset of point at a given distance
	point& offset(double distance, double angle_of_direction) {
		this->x += distance * std::cos(angle_of_direction + point_h_pi / 2.0);
		if (std::abs(this->x) < point_h_tolerance) this->x = 0.0;
		this->y += distance * std::sin(angle_of_direction + point_h_pi / 2.0);
		if (std::abs(this->y) < point_h_tolerance) this->y = 0.0;
		return *this;
	}

	point& push_off(double distance, double angle_of_direction) {
		this->x += distance * std::cos(angle_of_direction);
		if (std::abs(this->x) < point_h_tolerance) this->x = 0.0;
		this->y += distance * std::sin(angle_of_direction);
		if (std::abs(this->y) < point_h_tolerance) this->y = 0.0;
		return *this;
	}

	static double distance(point a, point b) {
		return std::sqrt(pow(a.x - b.x, 2.0) + pow(a.y - b.y, 2.0));
	}

	//get the unit circle angle (0 deg <= return < 360 deg)
	static double angle(point from, point to) {
		double ret = std::asin((to.y - from.y) / distance(from, to));
		if (std::abs(to.x - from.x) < point_h_tolerance)
			return ret; 
		if (to.x < from.x)
			ret = point_h_pi - ret;
		else if (std::abs(to.y - from.y) >= point_h_tolerance && to.y < from.y)
			ret = point_h_pi * 2.0 + ret;
		return ret;
	}

	double x;
	double y;
};

double radians_to_degrees(double a) {
	return a * 180 / point_h_pi;
}

double degrees_to_radians(double a) {
	return a * point_h_pi / 180;
}

bool is_greater(double a, double b) {
	if (std::abs(a - b) < point_h_tolerance)
		return false;
	else return a > b;
}

bool is_equal(double a, double b) {
	return std::abs(a - b) < point_h_tolerance;
}

bool is_less(double a, double b) {
	if (std::abs(a - b) < point_h_tolerance)
		return false;
	else return a < b;
}

#endif
